"""PyArr model constants."""

CONVERT_TO_BOOL = (
    "Advanced",
    "analyticsEnabled",
    "approved",
    "authentication",
    "downloadAllowed",
    "downloading",
    "enable",
    "enableAuto",
    "enableAutomaticSearch",
    "enableColorImpairedMode",
    "enabled",
    "enableInteractiveSearch",
    "enableRss",
    "enableSsl",
    "errors",
    "fullSeason",
    "hasFile",
    "includeCustomFormatWhenRenaming",
    "includeHealthWarnings",
    "includeQuality",
    "installable",
    "installed",
    "isAbsoluteNumbering",
    "isAdmin",
    "isAvailable",
    "isDaily",
    "isDebug",
    "isDocker",
    "isLinux",
    "isMono",
    "isNetCore",
    "isNewMovie",
    "isOsx",
    "isPossibleSpecialEpisode",
    "isProduction",
    "isRepack",
    "isUserInteractive",
    "isWindows",
    "launchBrowser",
    "latest",
    "monitored",
    "negate",
    "onDelete",
    "onDownload",
    "onGrab",
    "onHealthIssue",
    "onRename",
    "onUpgrade",
    "proper",
    "proxyBypassLocalAddresses",
    "proxyEnabled",
    "qualityCutoffNotMet",
    "renameMovies",
    "replaceIllegalCharacters",
    "replaceSpaces",
    "required",
    "sceneSource",
    "searchOnAdd",
    "sendUpdatesToClient",
    "shouldMonitor",
    "showRelativeDates",
    "special",
    "supportsOnDelete",
    "supportsOnDownload",
    "supportsOnGrab",
    "supportsOnHealthIssue",
    "supportsOnRename",
    "supportsOnUpgrade",
    "supportsRss",
    "supportsSearch",
    "unknownErrors",
    "unknownWarnings",
    "unverifiedSceneNumbering",
    "updateAutomatically",
    "warnings",
)
